package cn.com.qmth.ac.service.impl;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.validation.Assertion;
import org.jasig.cas.client.validation.Cas20ServiceTicketValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;

import cn.com.qmth.ac.bean.RedirectParam;
import cn.com.qmth.ac.bean.StatusException;
import cn.com.qmth.ac.bean.SystemConstant;
import cn.com.qmth.ac.service.CommonService;
import cn.com.qmth.ac.util.ByteUtil;
import cn.com.qmth.ac.util.SHA256;
import cn.com.qmth.ac.util.ServletUtil;
import cn.com.qmth.ac.util.StringUtil;

@Service
public class CommonServiceImpl implements CommonService {

    // private final static Logger log =
    // LoggerFactory.getLogger(CommonServiceImpl.class);

    @Autowired
    private Environment environment;

    @Override
    public void redirect(RedirectParam param) {

        HttpServletResponse response = ServletUtil.getResponse();
        response.setHeader(SystemConstant.ACCESS_CONTROL_ALLOW_ORIGIN, SystemConstant.PATH_MATCH);

        try {
            if (param.getTicketValid()) {
                long timestamp = System.currentTimeMillis();
                String secretKey = param.getSecretKey();

                String joinStr = StringUtil.join(param.getLoginName(), param.getRootOrgId(), param.getAppId(),
                        timestamp, secretKey);
                byte[] bytes = SHA256.encode(joinStr);
                String token = ByteUtil.toHexAscii(bytes);

                String url = param.getReturnUrl() + "/admin/access?loginName=" + param.getLoginName() + "&orgId="
                        + param.getRootOrgId() + "&appId=" + param.getAppId() + "&timestamp=" + timestamp + "&token="
                        + token + "&from=cas" + "&scode=" + param.getSchoolCode();
                response.sendRedirect(url);
            } else {
                response.sendRedirect(param.getReturnUrl());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public RedirectParam authentication(String schoolCode) {
        HttpServletRequest request = ServletUtil.getRequest();
        String authserver = environment.getProperty("examcloud.cas.authserver." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authserver)) {
            throw new StatusException(schoolCode + "未配置cas.authserver");
        }
        String authlogin = environment.getProperty("examcloud.cas.authlogin." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authserver)) {
            throw new StatusException(schoolCode + "未配置cas.authlogin");
        }

        String authlogout = environment.getProperty("examcloud.cas.authlogout." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authlogout)) {
            throw new StatusException(schoolCode + "未配置cas.authlogout");
        }

        String ecLogin = environment.getProperty("examcloud.login." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(ecLogin)) {
            throw new StatusException(schoolCode + "未配置examcloud.login");
        }

        String appId = environment.getProperty("examcloud.app-id." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(appId)) {
            throw new StatusException(schoolCode + "未配置examcloud.app-id");
        }

        String rootOrgId = environment.getProperty("examcloud.root-org-id." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(rootOrgId)) {
            throw new StatusException(schoolCode + "未配置examcloud.root-org-id");
        }
        String secretKey = environment.getProperty("examcloud.secret-key." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(secretKey)) {
            throw new StatusException(schoolCode + "未配置examcloud.login");
        }

        RedirectParam rp = new RedirectParam();

        rp.setAppId(appId);
        rp.setRootOrgId(rootOrgId);
        rp.setSecretKey(secretKey);

        String tkt = request.getParameter("ticket");

        String host = environment.getProperty("examcloud.auth.center.host");
        if (StringUtils.isBlank(host)) {
            throw new StatusException("未配置examcloud.auth.center.host");
        }

        String loginUrlRoot = host + "/api/auth/login/" + schoolCode;
        String encodeLoginUrlRoot = null;
        try {
            encodeLoginUrlRoot = URLEncoder.encode(loginUrlRoot, StandardCharsets.UTF_8.toString());
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        String loginUrl = authlogin + SystemConstant.GET_UNKNOWN + SystemConstant.SERVICE_KEY + SystemConstant.GET_EQUAL
                + encodeLoginUrlRoot;

        String errUrlRoot = "/api/auth/error?rurl=" + loginUrl;
        if (Objects.isNull(tkt) || tkt.length() == 0) {
            rp.setTicketValid(false);
            rp.setReturnUrl(loginUrl);
        } else {// 校验ticket，获取用户信息
            // 发送请求
            Cas20ServiceTicketValidator ticketValidator = new Cas20ServiceTicketValidator(authserver);

            try {
                // 验证 ticket
                Assertion assertion = ticketValidator.validate(tkt, loginUrlRoot);

                // 获取用户信息
                String username = assertion.getPrincipal().getName();
                rp.setTicketValid(true);
                rp.setLoginName(username);
                rp.setReturnUrl(ecLogin);

            } catch (Exception e) {
                rp.setTicketValid(false);
                rp.setReturnUrl(errUrlRoot);
            }

        }

        return rp;
    }

    @Override
    public String logout(String schoolCode) {
        return getLogoutUrl(schoolCode);
    }

    private String getLogoutUrl(String schoolCode) {
        String authlogout = environment.getProperty("examcloud.cas.authlogout." + schoolCode.toLowerCase());
        if (StringUtils.isBlank(authlogout)) {
            throw new StatusException(schoolCode + "未配置cas.authlogout");
        }

        String host = environment.getProperty("examcloud.auth.center.host");
        if (StringUtils.isBlank(host)) {
            throw new StatusException("未配置examcloud.auth.center.host");
        }

        try {
            String loginUrlRoot = host + "/api/auth/login/" + schoolCode;
            String encodeLoginUrlRoot = URLEncoder.encode(loginUrlRoot, StandardCharsets.UTF_8.toString());
            return authlogout + SystemConstant.GET_UNKNOWN + SystemConstant.SERVICE_KEY + SystemConstant.GET_EQUAL
                    + encodeLoginUrlRoot;
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }
}
